<?php
declare(strict_types=1);
/**
 * @copyright Copyright (c) 2018, Roeland Jago Douma <roeland@famdouma.nl>
 *
 * @author Roeland Jago Douma <roeland@famdouma.nl>
 *
 * @license GNU AGPL version 3 or any later version
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

namespace OCP\Authentication\TwoFactorAuth;

use OCP\IUser;
use Symfony\Component\EventDispatcher\Event;

/**
 * @since 15.0.0
 */
class RegistryEvent extends Event {

	/** @var IProvider */
	private $provider;

	/** @IUser */
	private $user;

	/**
	 * @since 15.0.0
	 */
	public function __construct(IProvider $provider, IUser $user) {
		$this->provider = $provider;
		$this->user = $user;
	}

	/**
	 * @since 15.0.0
	 */
	public function getProvider(): IProvider {
		return $this->provider;
	}

	/**
	 * @since 15.0.0
	 */
	public function getUser(): IUser {
		return $this->user;
	}
}
